var system = require("../../../system");
const http = require("http");
const querystring = require('querystring');
var settings = require("../../../../config/settings");
const CtlBase = require("../../ctl.base");
const moment = require('moment');
const appconfig = system.getSysConfig();
class DeliverybillCtl extends CtlBase {
  constructor() {
    super("bizchance", CtlBase.getServiceName(DeliverybillCtl));
    this.materialService = system.getObject("service.bizchance.materialSve");
    this.schemeService = system.getObject("service.bizchance.schemeSve");
    this.logService = system.getObject("service.bizchance.statuslogSve");
    this.receiveUrl = "http://192.168.1.140:4011/api/receive/entService/springBoard";
    this.queueUrl = "http://192.168.1.128:4018/api/queueAction/producer/springBoard";
  }

  /*根据用户id获取商机信息，分页获取*/
  async findAndCountAll(pobj, qobj, req) {
    try {
      //TODO 根据业务员id获取该业务员所有交付单列表
      const rs = await this.service.findAndCountAll(pobj);
      let rarr = [];
      let results = rs.results.rows;
      results.forEach(element => {
        let robj = {};
        robj.id = element.id;//数据的id
        robj.deliverNumber = element.delivery_code;//交付单编号
        robj.businessName = element.product_name;//产品名称
        robj.businessType = element.product_code;//产品编码
        robj.sourceName = element.source_name;//渠道来源
        robj.source_name = element.source_name;//渠道来源--前端服务
        robj.channelNumber = element.source_number;//渠道编码
        robj.facilitator_name = element.facilitator_name;//服务商--前端服务
        robj.serviceName = element.facilitator_name;
        robj.baseInfo = element.delivery_info;//交付单详情
        robj.payStatus = element.delivery_info.payStatus;//交付状态
        robj.costPrice = this.FenToYuan(element.cost_price);//成本
        robj.settle_status=element.settle_status;
        robj.settlebill=element.settlebill;
        if (robj.businessName == '公司注册'){
          if (robj.baseInfo.isWhether == "是" || robj.baseInfo.isVirtual == "是"){//如果有刻章需求或者是虚拟地址
            robj.relatedProducts = '有';
          }
          else {
            robj.relatedProducts = '无';
          }
        }
        else if (robj.businessName == '云上园区注册') {
          robj.registeredType = element.delivery_info.registeredType;//园区注册时，根据该值跳界面
          if (robj.baseInfo.isWhether == "是") {//如果有刻章需求或者是虚拟地址
            robj.relatedProducts = '有';
          }
          else {
            robj.relatedProducts = '无';
          }
        }
        else {
          robj.relatedProducts = '无';
        };
        robj.payAmount = this.FenToYuan(element.selling_price);//交付单金额
        robj.deliverStatus = element.delivery_status;//交付单状态
        robj.contactsName = robj.baseInfo.contactsName;//联系人
        robj.contactsPhone = robj.baseInfo.contactsPhone;//联系电话
        robj.clerkName = element.salesman_name;//业务员名称
        robj.clerkPhone = element.salesman_phone;//业务员电话
        robj.createdTime = this.timeFormat(element.created_at);//交付单创建时间
        rarr.push(robj);
      });
      rs.results.rows = rarr;
      return system.getResult(rs);
    } catch (error) {
      return system.getResultError(error);
    }
  }

  /*根据交付单编号获取交付单详情和材料详情*/
  async dealInfo(pobj, qobj, req) {
    if (pobj.deliverNumber && pobj.deliverNumber != 'undefined') {
      try {
        var rs = await this.service.findInfoByDeliverCode(pobj);
        var ms = await this.materialService.findInfoByDeliverNumber(pobj);
        var rarr = {
          "baseInfo": {},
          "contactsInfo": {},
          "statusInfo": {},
          "companyInfo": {},
          "registeredInfo": {},
          "contributionInfo": {},
          "positionInfo": {},
          "managerInfo": {},
          "regInfo": {},
          "expressInfo": {}
        }
        if (rs) {
          rarr.id = rs.id;//数据的id
          rarr.baseInfo.deliverNumber = rs.delivery_code;//交付单编号
          rarr.baseInfo.businessName = rs.product_name;//产品名称
          rarr.baseInfo.businessType = rs.product_code;//产品编码
          rarr.baseInfo.channelSource = rs.source_name;//渠道名称
          rarr.baseInfo.channelNumber = rs.source_number;//渠道编码
          rarr.baseInfo.serviceCode = rs.service_address;//服务地区编码
          rarr.baseInfo.servicerCode = rs.facilitator_id;//服务商编码
          rarr.baseInfo.servicerName = rs.facilitator_name;//服务商名称
          rarr.baseInfo.clerkName = rs.salesman_name;//业务员名称
          rarr.baseInfo.clerkPhone = rs.salesman_phone;//业务员电话
          rarr.baseInfo.sellingPrice = this.FenToYuan(rs.selling_price);//价格
          rarr.baseInfo.costPrice = this.FenToYuan(rs.cost_price);//成本价
          rarr.baseInfo.deliverStatus = rs.delivery_status;//状态
          rarr.baseInfo.statusReason = rs.close_reason;//状态原因
          if (rarr.businessName == "云上园区注册") {
            rarr.baseInfo.serviceArea = appconfig.pdict.registeredParkDict[rarr.serviceCode];//服务地区名称
          }
          else {
            rarr.baseInfo.serviceArea = appconfig.pdict.registeredAreaDict[rarr.serviceCode];//服务地区名称
          }
          rarr.contactsInfo.contactsName = rs.delivery_info.contactsName;//联系人信息
          rarr.contactsInfo.contactsPhone = rs.delivery_info.contactsPhone;//联系电话信息
          if (rs.delivery_info.isWhether && rs.delivery_info.isWhether != 'undefined') {
            rarr.baseInfo.isWhether = rs.delivery_info.isWhether;//是否刻章
          }
          //财税服务
          if (rs.delivery_info.financial && rs.delivery_info.financial != 'undefined') {
            rarr.baseInfo.financial = rs.delivery_info.financial;
          }
          if (rs.delivery_info.payStatus && rs.delivery_info.payStatus != 'undefined') {
            rarr.baseInfo.payStatus = rs.delivery_info.payStatus;//支付状态
          }
          if (rs.delivery_info.officialSeal && rs.delivery_info.officialSeal != 'undefined') {
            rarr.baseInfo.officialSeal = rs.delivery_info.officialSeal;//公章扫描件
          }
          if (rs.delivery_info.isAdviser && rs.delivery_info.isAdviser != 'undefined') {
            rarr.baseInfo.isAdviser = rs.delivery_info.isAdviser;//是否分配顾问
          }
          if (rs.delivery_info.businessLicense && rs.delivery_info.businessLicense != 'undefined') {
            rarr.baseInfo.businessLicense = rs.delivery_info.businessLicense;//营业执照扫描件
          }
          if (rs.delivery_info.taxpayerType && rs.delivery_info.taxpayerType != 'undefined') {
            rarr.baseInfo.taxpayerType = rs.delivery_info.taxpayerType;//纳税人类型
          }
          if (rs.delivery_info.buyDuration && rs.delivery_info.buyDuration != 'undefined') {
            rarr.baseInfo.buyDuration = rs.delivery_info.buyDuration;//购买时长
          }
          if (rs.delivery_info.buyTime && rs.delivery_info.buyTime != 'undefined') {
            rarr.baseInfo.buyTime = rs.delivery_info.buyTime;//购买时间
          }
          if (rs.delivery_info.buyDuration && rs.delivery_info.buyDuration != 'undefined' && rarr.baseInfo.buyTime && rarr.baseInfo.buyTime != 'undefined') {
            var d = new Date(rarr.baseInfo.buyTime);
            rarr.baseInfo.endTime = this.oneYearPast(d,Number(rs.delivery_info.buyDuration));//到期时间
            var ss = moment();
            rarr.baseInfo.surplusDuration =this.diffDays(ss,rarr.baseInfo.endTime);//剩余时长
          }
          if (rs.delivery_info.isRenew && rs.delivery_info.isRenew != 'undefined') {
            rarr.baseInfo.isRenew = rs.delivery_info.isRenew;//是否自动续费
          }
          if (rs.delivery_info.whetherType && rs.delivery_info.whetherType != 'undefined') {
            rarr.baseInfo.whetherType = rs.delivery_info.whetherType;//刻章类型
          }
          if (rs.delivery_info.subjectType && rs.delivery_info.subjectType != 'undefined') {
            rarr.baseInfo.subjectType = rs.delivery_info.subjectType;//代理记账主体类型
          }
          rarr.baseInfo.createdTime = this.timeFormat(rs.created_at);
          rarr.baseInfo.updateTime = this.timeFormat(rs.updated_at)


          rarr.statusInfo.deliverStatus = rs.delivery_status;//状态
          rarr.statusInfo.statusReason = rs.close_reason;//状态原因
          rarr.companyInfo = ms.company_info;//公司详情
          rarr.registeredInfo = ms.registered_info;//注册信息
          rarr.contributionInfo = ms.contribution_info;//出资比例信息
          rarr.positionInfo = ms.position_info;//任职信息；公司注册和个人独资时取该值
          rarr.managerInfo = ms.position_info;//经营者信息，个体工商户的经营者信息取该值
          rarr.regInfo = ms.file_info;//注册文件信息
          rarr.expressInfo = ms.express_info;//邮寄信息
        }
        return system.getResult(rarr);
      }
      catch (error) {
        return system.getResultError(error);
      }
    }
    else {
      return system.getResultError("参数错误！");
    }
  }

  /*根据交付单编号更新交付单状态*/
  async updateStatusByDeliverCode(pobj, qobj, req) {
    if (pobj.deliverNumber && pobj.deliverNumber != 'undefined' && pobj.deliverStatus && pobj.deliverStatus != 'undefined') {
      try {
        //调商城接口，回传状态信息，放队列
        var rc = system.getObject("util.execClient");
        var requrl = this.queueUrl;
        var params = {
          "actionType": "produceData",
          "actionBody": {
            "pushUrl": this.receiveUrl,
            "actionType": "updateOrderStatus",
            "identifyCode": "ic-manage",
            "messageBody": {
              "orderNum": pobj.businessMode,//订单编码
              "status": appconfig.pdict.ddyDict[pobj.deliverStatus]
              //"deliverContent":pobj
            }
          }
        }
        var rtn = await rc.execPost(params, requrl);
        var j = JSON.parse(rtn.stdout);
        if (j.status == 1) {
          await this.service.updateStatusByDeliverCode(pobj);
          var pInfo = this.service.findInfoByDeliverCode(pobj);
          if (pInfo){
            var logInfo = {
              "flowType":"DELIVERY",
              "flowId":pInfo.id,
              "statusCode":pobj.deliverStatus,
              "clerkName":pInfo.salesman_name,
              "clerkId":pInfo.salesman_id
            }
            if (pInfo.statusReason && pInfo.statusReason != 'undefined'){
              logInfo.statusReason = pInfo.statusReason;
            }
            await this.logService.insertInfo(logInfo);//插入状态日至表
          }
          return system.getResult("更新成功！");
        }
        else {
          return system.getResultError("更新状态失败！");
        }
      }
      catch (error) {
        return system.getResultError(error);
      }
    }
    else {
      return system.getResultError("参数错误!");
    }
  }

  async UpdateofficialSealByDeliverCode(pobj, qobj, req) {//更新营业执照扫描件和公章扫描件
    if ((pobj.deliverNumber && pobj.deliverNumber != 'undefined' && ((pobj.baseInfo.officialSeal && pobj.baseInfo.officialSeal != 'undefined') || (pobj.baseInfo.businessLicense && pobj.baseInfo.businessLicense != 'undefined')))) {
      try {
        var rs = await this.service.findInfoByDeliverCode(pobj)//先得到交付单详情
        if (rs && rs != 'undefined') {
          if (pobj.baseInfo.officialSeal && pobj.baseInfo.officialSeal != 'undefined') {
            rs.delivery_info.officialSeal = pobj.baseInfo.officialSeal;
          }
          if (pobj.baseInfo.businessLicense && pobj.baseInfo.businessLicense != 'undefined') {
            rs.delivery_info.businessLicense = pobj.baseInfo.businessLicense;
          }
          //回传信息给庄冰，放队列
          var rc = system.getObject("util.execClient");
          var requrl = this.queueUrl;
          var params = {
            "actionType": "produceData",
            "actionBody": {
              "pushUrl": this.receiveUrl,
              "actionType": "updateOrderStatus",
              "identifyCode": "ic-manage",
              "messageBody": {
                "orderNum": pobj.businessMode,//订单编码
                "status": appconfig.pdict.ddyDict[rs.delivery_status],
                "deliverContent": rs.delivery_info
              }
            }
          }
          var rtn = await rc.execPost(params, requrl);
          var j = JSON.parse(rtn.stdout);
          console.log("RRRRRRRRRRRRRR");
          console.log(JSON.stringify(j));
          if (j.status == 1) {
            //更新详情
            var uarr = {
              "deliverNumber": pobj.deliverNumber,
              "baseInfo": rs.delivery_info
            };
            await this.service.updateInfoByDeliverCode(uarr);
            return system.getResult("更新成功");
          }
          else {
            return system.getResultError("更新失败！");
          }
        }
        else {
          return system.getResultError("数据不存在！");
        }

      }
      catch (error) {
        console.log("##########");
        console.log(error);
        return system.getResultError(error);
      }
    }
    else {
      console.log("参数错误" + pobj);
      return system.getResultError("参数错误！");
    }
  }

  async updateExpressInfoByDeliverCode(pobj, qobj, req) {//更新邮寄信息
    if (pobj.deliverNumber && pobj.deliverNumber != 'undefined' && pobj.expressInfo && pobj.expressInfo != 'undefined') {
      try {
        var deliverInfo = await this.service.findInfoByDeliverCode(pobj)//先得到交付单详情
        if (deliverInfo && deliverInfo != 'undefined') {
          //回传信息给庄冰，放队列
          var rc = system.getObject("util.execClient");
          var requrl = this.queueUrl;
          var params = {
            "actionType": "produceData",
            "actionBody": {
              "pushUrl": this.receiveUrl,
              "actionType": "updateOrderStatus",
              "identifyCode": "ic-manage",
              "messageBody": {
                "orderNum": pobj.businessMode,//订单编码
                "status": appconfig.pdict.ddyDict[deliverInfo.delivery_status],
                "deliverContent": rs.delivery_info
              }
            }
          }
          var j = JSON.parse(rtn.stdout);
          if (j.status == 1) {
            await this.materialService.updateInfoByDeliverNumber(pobj);
            return system.getResult("更新成功");
          }
          else {
            return system.getResultError("更新失败！");
          }

        }
        else {
          return system.getResultError("数据不存在！");
        }

      }
      catch{
        return system.getResultError(error);
      }
    }
    else {
      return system.getResultError("参数错误！");
    }
  }

  /*根据交付单编号更新交付单详情*/
  async updateInfoByDeliverCode(pobj, qobj, req) {//
    if (pobj.deliverNumber && pobj.deliverNumber != 'undefined') {
      try {
        var rs = await this.service.findInfoByDeliverCode(pobj)//先得到交付单详情
        var baseInfo = rs.delivery_info;
        if (pobj.contactsInfo.contactsName && pobj.contactsInfo.contactsName != 'undefined') {
          baseInfo.contactsName = pobj.contactsInfo.contactsName;
        }
        if (pobj.contactsInfo.contactsPhone && pobj.contactsInfo.contactsPhone != 'undefined') {
          baseInfo.contactsPhone = pobj.contactsInfo.contactsPhone;
        }
        var parms = {
          "deliverNumber": pobj.deliverNumber
        }
        if (rs.product_code == "psCase") {
          parms.deliverStatus = "checking";//状态为园区入驻审核
          baseInfo.payStatus = "已交付";
        }
        else if (rs.product_code == "companyCase") {
          parms.deliverStatus = "auditing";//状态为工商审核环节
          baseInfo.payStatus = "已交付";
        }
        parms.baseInfo = baseInfo;
        await this.service.updateInfoByDeliverCode(parms);
        //更新材料信息
        await this.materialService.updateInfoByDeliverNumber(pobj);
        //return system.getResult("操作成功！");
        //传给庄冰，tongbu
        baseInfo.clerkName = rs.salesman_name;//业务员名称
        baseInfo.clerkPhone = rs.salesman_phone;
        pobj.baseInfo = baseInfo;
        var rc = system.getObject("util.execClient");
        var requrl = this.receiveUrl;
        var params = {
          "actionType": "updateOrderStatus",
          "actionBody": {
            "orderNum": pobj.deliverNumber,//订单编码
            //"status":appconfig.pdict.ddyDict["collecting"],
            "deliverContent": pobj
          }
        }
        if (rs.product_code == "psCase") {
          params.actionBody.status = appconfig.pdict.ddyDict["checking"];
        }
        else if (rs.product_code == "companyCase") {
          params.actionBody.status = appconfig.pdict.ddyDict["auditing"];
        }
        else {
          params.actionBody.status = appconfig.pdict.ddyDict[rs.delivery_status];
        }
        var rtn = await rc.execPost(params, requrl);
        var j = JSON.parse(rtn.stdout);
        if (j.status == 1) {
          return system.getResult("操作成功！");
        }
        else {
          return system.getResultError("更新交付单状态出错！");
        }
      }
      catch (error) {
        return system.getResultError(error);
      }
    }
    else {
      return system.getResultError("参数错误！");
    }

  }

  async settleApply(p,q,req){
      let ids=p.ids
      let rtn=await this.service.settleApply(ids,p.userid,p.username)
      return system.getResult({})
  }
  /*根据商机编号插入交付单信息*/
  async insertInfo(pobj, qobj, req) {//队列的时候用
    if (pobj.businessMode && pobj.businessMode != 'undefined' && pobj.schemeNumber && pobj.schemeNumber != 'undefined'
      && pobj.deliverNumber && pobj.deliverNumber != 'undefined') {
      try {
        pobj.deliverStatus = "received";
        if (pobj.businessMode && pobj.businessMode != 'undefined') {//如果有商机编号，需要取方案编号
          var schemeInfo = await this.schemeService.findInfoByDemandCode(pobj);
          if (schemeInfo && schemeInfo != 'undefined') {
            pobj.schemeNumber = schemeInfo.scheme_number;
          }
        }
        //更新状态给庄冰
        var rc = system.getObject("util.execClient");
        var requrl = this.queueUrl;
        var params = {
          "actionType": "produceData",
          "actionBody": {
            "pushUrl": this.receiveUrl,
            "actionType": "updateOrderStatus",
            "identifyCode": "ic-manage",
            "messageBody": {
              "orderNum": pobj.businessMode,//订单编码
              "status": 20,//已接单
              "deliverContent": pobj
            }
          }
        }
        var rtn = await rc.execPost(params, requrl);
        var j = JSON.parse(rtn.stdout);
        if (j.status == 1) {
          var res = await this.service.insertInfo(pobj);
          var mobj = {
            "deliverId": res.id,
            "deliverNumber": pobj.deliverNumber
          }
          var logInfo = {
            "flowType":"DELIVERY",
            "flowId":res.id,
            "statusCode":res.delivery_status,
            "statusReason":res.close_reason,
            "clerkName":res.salesman_name,
            "clerkId":res.salesman_id
          }
          await this.logService.insertInfo(logInfo);//插入状态日至表

          if (pobj.companyInfo && pobj.companyInfo != 'undefined') {
            mobj.companyInfo = pobj.companyInfo;
          }
          await this.materialService.insertInfo(mobj);
          return system.getResult("插入成功！");
        }
        else {
          return system.getResultError("插入数据失败！");
        }

      }
      catch (error) {
        return system.getResultError(error);
      }
    } else {
      return system.getResultError("参数错误!");
    }
  }



  timeFormat(date) {
    let localTime = moment.utc(date).toDate();
    localTime = moment(localTime).format("YYYY-MM-DD");
    return localTime;
  }

  //元转分 - 解决精度问题 yuan:要转换的钱，单位元； digit：转换倍数
  YuanToFen(yuan, digit) {
    var m = 0,
    s1 = yuan.toString(),
    s2 = digit.toString();
    try { m += s1.split(".")[1].length } catch (e) { }
    try { m += s2.split(".")[1].length } catch (e) { }
    return Number(s1.replace(".", "")) * Number(s2.replace(".", "")) / Math.pow(10, m)
  }

  FenToYuan(fen) {
    var num = fen;
    num = fen * 0.01;
    num += '';
    var reg = num.indexOf('.') > -1 ? /(\d{1,3})(?=(?:\d{3})+\.)/g : /(\d{1,3})(?=(?:\d{3})+$)/g;
    num = num.replace(reg, '$1');
    num = this.toDecimal2(num);
    return num;
  }

  toDecimal2(x) {
    var f = parseFloat(x);
    if (isNaN(f)) {
      return false;
    }
    var f = Math.round(x * 100) / 100;
    var s = f.toString();
    var rs = s.indexOf('.');
    if (rs < 0) {
      rs = s.length;
      s += '.';
    }
    while (s.length <= rs + 2) {
      s += '0';
    }
    return s;
  }

  //time时间n年后的的前一天
  oneYearPast(time,n) {
    //var time=new Date();
    var year = time.getFullYear() + n;
    var d = time.setFullYear(year);
    var date = time.getDate(d) -1;
    var xx = time.setDate(date);
    return moment( new Date(xx)).format('yyyy-MM-DD');
  }

  //计算俩日期相差多少天
  diffDays(time1,time2){
    var m1 = moment(time1);
    var m2 = moment(time2);
    return m2.diff(m1,'day');
  }
}
module.exports = DeliverybillCtl;

// var task = new DeliverybillCtl();
// var d = new Date("2020-06-05");
// var oneYear = task.oneYearPast(d);
// var ss = moment();
// var diff = task.diffDays(ss,oneYear);
// console.log(oneYear);
// console.log(diff);
// var obj={"deliverNumber":"1111115","baseInfo":"","deliverInfo":{"isVirtual": "否", "isWhether": "是", "payStatus": "已交付", "contactsName": "张三", "contactsPhone": "13800138000"},"companyInfo":{"isWhether": "no", "addressType": "practical", "fullAddress": "gsgsgsgsgsg", "taxpayerType": "smallScaleTaxpayer", "businessScope": "sgsgsdgsdgd", "engagedIndustry": "culturalMedia", "companyProperties": "limitedLiabilityCompany"},
//           "registeredInfo":{"registeredCapital":"500","registeredDate":"20","reserveProportion":"15"},"contributionInfo":{"contribution_info":[{"shareholderName":"王五","contributionAmount":"200","contributionProportion":"40","IdentificationNumber":"1111111","phoneNumber":"1111111","contactAddress":"北京"},
//           {"shareholderName":"王六","contributionAmount":"200","contributionProportion":"40","IdentificationNumber":"1111111","phoneNumber":"1111111","contactAddress":"北京"},
//           {"shareholderName":"王七","contributionAmount":"100","contributionProportion":"20","IdentificationNumber":"1111111","phoneNumber":"1111111","contactAddress":"北京"}]}};
// task.updateInfoByDeliverCode(obj,{},{}).then(d=>{
//   console.log(d);
//   //console.log(d[0].analyse);

// })
